// Copyright (C) 1999 Dominic Letourneau

#ifndef _PATHLIST_H_
#define _PATHLIST_H_


#include "Node.h"
#include <queue>

/** The PathList class. */
class PathList : public Node {

public:

   /** The constructor with a nodeName and a parameter set 
       parameters:   PATH     ->The path 
                     EXTENSION->The file extension we want to find
    */
   PathList (string nodeName, ParameterSet params);

   /** The getOutput method overloader from Node */
   virtual ObjectRef getOutput (int output_id, int count);
   
   void specificInitialize() {processCount=-1;Node::specificInitialize();}

   void reset() {processCount=-1;Node::reset();}

private:
   int processCount;

   /// The filename Output (PATH + FILENAME)
   ObjectRef fullName;

   ///The filename only (FILENAME)
   ObjectRef shortName;

   ///The outputID of the full path
   int fullPathID;

   ///The outputID of the file name
   int fileNameID;

   /// The filename queue
   queue<string> fileQueue;
 
   /// The path of the directory listing
   string path;

   /// The extension of the file we want to get
   string extension;

   /// The default constructor
   PathList() {
      throw new NodeException (NULL,"The default constructor should not be called from PathList",__FILE__,__LINE__);
   }

   ///Reads the path given in a parameter (using the file extension)
   void ReadPath();

   ///Removes the path from a full file name
   string removePath(const string &fullName);
};

#endif
